package de.intevation.flys.client.server;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import de.intevation.artifacts.common.ArtifactNamespaceContext;
import de.intevation.artifacts.common.utils.ClientProtocolUtils;
import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.artifacts.httpclient.exceptions.ConnectionException;
import de.intevation.artifacts.httpclient.http.HttpClient;
import de.intevation.artifacts.httpclient.http.HttpClientImpl;
import de.intevation.artifacts.httpclient.http.response.DocumentResponseHandler;

import de.intevation.flys.client.shared.exceptions.ServerException;
import de.intevation.flys.client.shared.model.Artifact;
import de.intevation.flys.client.client.services.GetArtifactService;


/**
 * This service provides a method that returns an artifact based on its
 * identifier.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class GetArtifactServiceImpl
extends      RemoteServiceServlet
implements   GetArtifactService
{
    private static final Logger logger =
        Logger.getLogger(GetArtifactServiceImpl.class);


    public static final String ERROR_DESCRIBE_ARTIFACT =
        "error_describe_artifact";

    public static final String XPATH_RESULT = "/art:result/text()";

    public static final String OPERATION_FAILURE = "FAILED";


    public Artifact getArtifact(
        String url,
        String locale,
        String uuid,
        String hash)
    throws ServerException
    {
        logger.info("GetArtifactServiceImpl.getArtifact");

        Document describe = ClientProtocolUtils.newDescribeDocument(
            uuid, hash, true);

        HttpClient client = new HttpClientImpl(url, locale);

        try {
            Document description = (Document) client.describe(
                new de.intevation.artifacts.httpclient.objects.Artifact(
                    uuid, hash),
                describe,
                new DocumentResponseHandler());

            if (description == null) {
                throw new ServerException(ERROR_DESCRIBE_ARTIFACT);
            }

            String result = XMLUtils.xpathString(
                description,
                XPATH_RESULT,
                ArtifactNamespaceContext.INSTANCE);

            if (result == null || !result.equals(OPERATION_FAILURE)) {
                return (Artifact) new FLYSArtifactCreator().create(description);
            }
        }
        catch (ConnectionException ce) {
            logger.error(ce, ce);
        }

        throw new ServerException(ERROR_DESCRIBE_ARTIFACT);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
